import pandas as pd
import matplotlib.pyplot as plt
from scipy.stats import pearsonr
import numpy as np
from sklearn import datasets, linear_model
from sklearn.linear_model import LinearRegression
import statsmodels.api as sm
from scipy import stats

# Load data
df = pd.read_csv("./data/ing_pairplots.csv", encoding='latin-1')

# Get the list of all columns except 'Stage'
columns_to_plot = df.columns.drop('Stage (m)')

# Create subplots
num_plots = len(columns_to_plot)
fig, axes = plt.subplots(nrows=1, ncols=num_plots, figsize=(6*num_plots, 6), sharey=True)

# Plot scatterplot for each variable against 'Stage' and annotate with p-value and R2 value
for i, col in enumerate(columns_to_plot):
    # Drop rows with NaN values in the two columns being compared
    data = df[[col, 'Stage (m)']].dropna()

    # Scatterplot
    axes[i].scatter(data[col], data['Stage (m)'], alpha=0.5)
    axes[i].set_xlabel(col, fontsize=22)

    # Increase tick label fontsize
    axes[i].tick_params(axis='both', which='major', labelsize=20)

    # Calculate correlation coefficient and p-value
    corr_coef, p_value = pearsonr(data[col], data['Stage (m)'])

    # Calculate R-squared value
    x = data[col]
    y = data['Stage (m)']
    r_squared = np.corrcoef(x, y)[0, 1]**2

    # Convert p-value to scientific notation
    p_value_sci = "{:.2e}".format(p_value)

    # Annotate the subplot with the p-value and R2 value
    axes[i].annotate(f"p-value: {p_value_sci}\nR2: {r_squared:.4f}", xy=(0.5, 0.83),
                     xycoords='axes fraction', fontsize=20)

# Set a common y-axis label with increased fontsize
axes[0].set_ylabel('Stage (m)', fontsize=22)

# Adjust layout and display the plot
plt.tight_layout()
plt.show()

## Calculate p-values between all variables to check independence
results = df.corr(method='pearson')
p_values = df.corr(method=lambda x, y: stats.pearsonr(x, y)[1])

# Display the DataFrame with p-values
print(p_values)

alpha=0.05/18
print(f"Bonferroni-corrected alpha: {alpha}")

"""Air temperature (at all lag steps), upward, downward, and net solar radiation, ice sheet, proglacial, and full watershed albedo, and ice sheet and full watershed snow cover were significantly correlated with the hybrid stage after applying a Bonferroni correction to alpha. Precipitation had no significant correlation, likely because of the coarse resolution of the remotely sensed data.

The strongest independent predictor from each cluster of correlated variables was selected for the multivariate model. These variables were air temperature and ablation zone albedo.
"""

# Remove NAN rows
df = df.dropna()

# Build multivariate linear regression model
X = df[['Air Temperature (°C)', 'Ice Sheet Albedo']]
y = df['Stage (m)']

X2 = sm.add_constant(X)
est = sm.OLS(y, X2)
est2 = est.fit()
print(est2.summary())
print(est2.summary2().tables[1])